/**************************************************************************
 *    file:         vtDef.h                                               *
 *    create:       June 3, 2000                                          *
 *    copyright:    @ 2000, by Zhaohui Xing                               *
 *    e-mail:       xinghh@yahoo.com                                      * 
 *    description:  definition of global data type, constants and         *
 *                  marcos.                                               *  
 **************************************************************************/
#ifndef __VOICETALK_DEF_H__
#define __VOICETALK_DEF_H__

#include <windows.h>            
#include <windowsx.h>
#include <winsock.h>
#include <nspapi.h>
#include <svcguid.h>
#include <wsipx.h>
#include <wsnetbs.h>
  

/**************************************************************************
 *  Voive Talk constants definitions                                      *
 **************************************************************************/
#define  VT_MAX      0x1000

#define  VT_TEXT     0x00                // Text message 
#define  VT_WAVE1M08 0x01                // Voice: 11.025 kHz, Mono, 8-bit  
#define  VT_WAVE1S08 0x02                // Voice: 11.025 kHz, Stereo, 8-bit
#define  VT_WAVE1M16 0x03                // Voice: 11.025 kHz, Mono, 16-bit 
#define  VT_WAVE1S16 0x04                // Voice: 11.025 kHz, Stereo, 16-bit 
#define  VT_WAVE2M08 0x05                // Voice: 22.05 kHz, Mono, 8-bit 
#define  VT_WAVE2S08 0x06                // Voice: 22.05 kHz, Stereo, 8-bit  
#define  VT_WAVE2M16 0x07                // Voice: 22.05 kHz, Mono, 16-bit 
#define  VT_WAVE2S16 0x08                // Voice: 22.05 kHz, Stereo, 16-bit 
#define  VT_WAVE4M08 0x09                // Voice: 44.1 kHz, Mono, 8-bit  
#define  VT_WAVE4S08 0x0A                // Voice: 44.1 kHz, Stereo, 8-bit  
#define  VT_WAVE4M16 0x0B                // Voice: 44.1 kHz, Mono, 16-bit 
#define  VT_WAVE4S16 0x0C                // Voice: 44.1 kHz, Stereo, 16-bit


/**************************************************************************
 *  Voive Talk wave sample frequency constants                            *
 **************************************************************************/
const DWORD VTWAVEFREQ[13] = {0,    
                              11025,    
						      11025,
                              11025,    
							  11025,    
							  22050,
                              22050,   
							  22050,   
							  22050,
                              44100,   
							  44100,   
							  44100,
                              44100};


/**************************************************************************
 *  Voive Talk wave channel constants, 1: Mono, 2: Stereo                 *
 **************************************************************************/
const WORD VTWAVECHANNEL[13] = {0, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2};


/**************************************************************************
 *  Voive Talk wave sample bit constants, 8: 8 bit, 16: 16 bit            *
 **************************************************************************/
const WORD VTWAVEBITS[13] = {0, 8, 8, 16, 16, 8, 8, 16, 16, 8, 8, 16, 16};


/**************************************************************************
 *  Voive Talk UID constants                                              *
 **************************************************************************/
#ifdef __TALKSERVER_COMPILE__
#define  VT_NETID    0x5607              // Network order
#else
#define  VT_NETID    0x0756 
#endif
#define  VT_DNSID    0x555               // TCP port


/**************************************************************************
 * Specific Windows Messages definition.                                  *
 **************************************************************************/
#define VTM_CONNECTED             2200
#define VTM_DATAREADY             2201 
#define VTM_READYFORWRITE         2202 
#define VTM_RECORDREADY           2203 


/**************************************************************************
 *  Voive Talk socket message data unit definitions                       *
 **************************************************************************/
typedef struct _VTMSG  
{
    BYTE  m_ucIdentity;    // Identifies the message  
    BYTE  m_ucCmd;         // message command
    BYTE  m_ucMsgType;     // The message type 
    UINT  m_lLength;       // size of the message block 
	BYTE  m_pData[VT_MAX]; // message data
} VTMSG, *LPVTMSG;

#define SIZEVTMSG      sizeof(VTMSG)     // The size of VTMSG structure.

#define SIZEVTMSGHDR  4*sizeof(BYTE)+sizeof(long) // The sum of m_ucIdentity,
                                                  // m_ucMsgType, m_lLength,
                                                  // m_ucCmd's sizes in VTMSG
                                                  // structure

#define REALLEN(x)     lstrlen(x) + 1    // The string length, including the 
                                         // terminating NULL char
/**************************************************************************
 * Voice Talk socket message commands                                     *
 **************************************************************************/
#define VTCMD_REGNAME             1
#define VTCMD_MSGDATA             2
#define VTCMD_REQSESSION          3
#define VTCMD_SESSIONREQRESP      4
#define VTCMD_SESSIONCLOSE        5
#define VTCMD_DEREGNAME           6

/**************************************************************************
 * The Voice Talk Identity for socket message                             *
 **************************************************************************/
#define VT_IDENTITY               0xCC   


/**************************************************************************
 *  Voive Talk socket structure for storing the socket information        *
 **************************************************************************/
typedef struct _VTSOCKET
{
    SOCKET           m_hSock;           // The main socket handle
    struct sockaddr  m_sockAddr;        // Address structure
	char             m_Reserved[10];    // Generic sockaddr struct is not 
	                                    // big enough for all addresses (
	                                    // namely netbios addresses) so 
	                                    // adding some space at the end.
    int              m_iStatus;         // Socket's status
	int              m_iAddrLen;        // Address length
    int              m_iSockType;       // Socket type (from socket())
    int              m_iProtocol;       // Socket protocol (from socket())
    int              m_iConnects;       // Number of current connections on 
	                                    // this socket
    char             m_sUserName[16];   // User name
    SOCKET           m_hPeerSock;       // Associated peer socket which this 
	                                    // socket is in session with

#ifdef __TALKSERVER_COMPILE__
    LPTSTR           m_lpProtocolName;  // Pointer to sccker protocol name 
    int              m_iServSockIndex;  // Index to array of server sockets
#endif 
} VTSOCKET, *LPVTSOCKET;

/**************************************************************************
 * VTSOCKET structure status options                                      *
 **************************************************************************/
#define VTSOCKET_INIT        1
#define VTSOCKET_LISTENING   2
#define VTSOCKET_ACCEPTING   3
#define VTSOCKET_CLOSED      4
#define VTSOCKET_CONNECTED   5
#define VTSOCKET_AVAILABLE   6
#define VTSOCKET_INSESSION   7
#define VTSOCKET_REQSESSION  8


/**************************************************************************
 *  SAP packet definition                                                 *
 **************************************************************************/
typedef struct _VTSAPPACK
{
    short    m_iOperation;    // 1 = Request.
	                          // 2 = Response.
	                          // 3 = Get nearest request,
	                          // 4 = Get nearest response.
    short    m_iService;      // kind of service
    char     m_sServName[48]; // the server name
    char     m_sNetID[4];     // network number
    char     m_sNode[6];      // node number
    char     m_sSocket[2];    // socket number
    short    m_iHops;         // Number of hops to server
} VTSAPPACK, *LPVTSAPPACK; 


/**************************************************************************
 *  Global function prototypes                                            *
 **************************************************************************/
extern "C"
{
void Ascii2Hex(char* pSrc, char* pDest, int iLen);
BYTE AsciiByte2Hex(char ch);
BOOL SendVTMessage(SOCKET sock, LPVTMSG sendBuf);
BOOL RecvVTMessage(LPVTSOCKET lpvtSock, LPVTMSG recvBuf);
#ifdef __TALKSERVER_COMPILE__
BOOL BuildServer(HWND hWnd, LPVTSOCKET pServSock, LPPROTOCOL_INFO lpProtBuf);
#endif 
}

#endif