/**************************************************************************
 *    file:         vtFunc.h                                              *
 *    create:       June 4, 2000                                          *
 *    copyright:    @ 2000, by Zhaohui Xing                               *
 *    e-mail:       xinghh@yahoo.com                                      * 
 *    description:  implementation of some fundmental functions           *
 **************************************************************************/
#include "stdafx.h"
#include "vtDef.h"

/**************************************************************************
 * Converts ascii string to network order hex                             * 
 **************************************************************************/
void Ascii2Hex(char* pSrc, char* pDest, int iLen)
{
    char * sPTR;
    int iLoop;
	
    sPTR = pSrc;

    for(iLoop = 0; iLoop < iLen; iLoop++)
    {
       // Put 1st ascii byte in the high bites.
       *pDest = AsciiByte2Hex(*sPTR++) << 4;  
	   // Add 2nd ascii byte to above.
       *pDest += AsciiByte2Hex(*sPTR++);      
	   pDest++;
    }
}


/**************************************************************************
 * Converts ascii byte to numeric                                         *
 **************************************************************************/
BYTE AsciiByte2Hex(char ch)
{
	// Process numerals
    if(ch >= '0' && ch <= '9') 
		return (ch - '0');        
    // Process capitol hex digits
	if(ch >= 'A' && ch <= 'F') 
		return (ch - 'A' + 0xA);  
	// Process small hex digits
    if(ch >= 'a' && ch <= 'f') 
		return (ch - 'a' + 0xA);  
    
	//Nothing
	return(255);
}


/**************************************************************************
 * Send the message to Voice Talk clients                                 *
 **************************************************************************/
BOOL SendVTMessage(SOCKET sock, LPVTMSG sendBuf)
{
    int byteAll;
	int byteSent;
	int sendingSize;

    // Initialize 
    byteSent = 0;                    
	byteAll = sendBuf->m_lLength;
    
	while(byteSent < byteAll)
    {
        // Send message to the client. If not all message has been
		// sent, then continue sending until the whole message is gone.
        if((sendingSize = (int)send(sock, (char *)sendBuf + byteSent, 
			        byteAll - byteSent, 0)) == SOCKET_ERROR)
		{
		     // Error
			 return FALSE;
	    }
        byteSent += sendingSize;
    }

    return TRUE;
}


/**************************************************************************
 * Receive the message from Voice Talk clients                            *
 **************************************************************************/
BOOL RecvVTMessage(LPVTSOCKET lpvtSock, LPVTMSG recvBuf)
{
    int byteLeft; 
	int byteAll;
	int msgSize;
	int byteRead;
	LPVTMSG pData = new VTMSG;
	memset((void*)pData, 0, SIZEVTMSG);

	// Message mode protocol, read the message data
    if (lpvtSock->m_iSockType == SOCK_SEQPACKET)
    {
		// Set full data size
        byteLeft = sizeof(*recvBuf);
    }
    else // Stream mode protocol, read header information
    {
		// Set data size to header information block
        byteLeft = SIZEVTMSGHDR;
    }


    // First time to read message to determine the message type information 
	// and size data
    if((byteAll = (int)recv(lpvtSock->m_hSock, (char *)pData, byteLeft, 0)) 
		== SOCKET_ERROR)
    {
        //ERROR 
        return FALSE;
    }

    // Check the message identity to make sure it is valid message
    if(pData->m_ucIdentity != VT_IDENTITY)
    {
        //Wrong message
        return FALSE;
    }

    msgSize = pData->m_lLength;  // The total size of the message
    byteLeft = msgSize - byteAll;  // The bytes left to read

    while(byteAll < msgSize)
    {
        // Read the message from the stream sockets
        if((byteRead = (int)recv(lpvtSock->m_hSock,
                             (char *)pData + byteAll,
                             byteLeft,
                             0)) == SOCKET_ERROR)
        {
            if ((int)WSAGetLastError() != WSAEWOULDBLOCK)
            {
                // ERROR
                return FALSE;
            }
        }
        byteAll += byteRead;
        byteLeft -= byteRead;
    }

	memcpy((void*)recvBuf, (void*)pData, SIZEVTMSG);

	delete []pData;

    return TRUE;
}


/**************************************************************************
 * Intialize the server sockets                                           *
 **************************************************************************/
#ifdef __TALKSERVER_COMPILE__
BOOL BuildServer(HWND hWnd, LPVTSOCKET pServSock, LPPROTOCOL_INFO lpProtBuf)
{
    CSADDR_INFO CSABuf[10];
    DWORD dwCSABufsize = sizeof(CSABuf);
    GUID guidNW = SVCID_NETWARE(VT_NETID);
    GUID guidDNS = SVCID_TCP(VT_DNSID); 
    LPSOCKADDR_NB lpNBaddr;
    char strProts[16]; 


    //Open the socket with the protocol defined by PROTOCOL_INFO structure
    if((pServSock->m_hSock = socket(lpProtBuf->iAddressFamily,
                                    lpProtBuf->iSocketType,
                                    lpProtBuf->iProtocol)) == SOCKET_ERROR)
    {
       // ERROR
       return FALSE;  // This will reuse the current ServSocks structure
    }

    // Set the server socket data structure
    pServSock->m_iStatus = VTSOCKET_INIT;
    pServSock->m_iSockType = lpProtBuf->iSocketType;
    pServSock->m_iProtocol = lpProtBuf->iProtocol;
    pServSock->m_iConnects = 0;
    pServSock->m_lpProtocolName = lpProtBuf->lpProtocol;
	
	//Check the socket rotocol and set socket address
	//address length
    switch (lpProtBuf->iAddressFamily)
    {
       case AF_IPX:
           // Use SAP/Bindery Name Space
           if(GetAddressByName(0, &guidNW, "VOICE_TALK_SERVER_IPX", NULL,    
                               RES_SERVICE, NULL, CSABuf, &dwCSABufsize,  
                               NULL, NULL) < 1)    
           {
               // Error--cleanup
               closesocket(pServSock->m_hSock);
               return FALSE;
           }

           // Copy sockaddr info and addresslength
           memcpy(&pServSock->m_sockAddr, CSABuf[0].LocalAddr.lpSockaddr, 
			       sizeof(struct sockaddr));
           pServSock->m_iAddrLen = CSABuf[0].LocalAddr.iSockaddrLength;
		   strcpy(strProts, "AF_IPX");
           break;

       case AF_INET:
           // Use DNS (TCP/IP protocol) Name Space
           if (GetAddressByName(0, &guidDNS, "VOICE_TALK_SERVER_TCP/IP", NULL,
                                RES_SERVICE, NULL, CSABuf, &dwCSABufsize,
                                NULL, NULL) < 1)   
           {
               // Error -- cleanup
               closesocket(pServSock->m_hSock);
               return FALSE;
           }
           // Copy sockaddr info and addresslength
           memcpy(&pServSock->m_sockAddr, CSABuf[0].LocalAddr.lpSockaddr, 
			       sizeof(struct sockaddr));
           pServSock->m_iAddrLen = CSABuf[0].LocalAddr.iSockaddrLength;
		   strcpy(strProts, "AF_INET");
           break;

       case AF_NETBIOS:
           // no netbios name space provider so we will simulate GetAddressByName() results
           lpNBaddr = (LPSOCKADDR_NB)&pServSock->m_sockAddr;
           SET_NETBIOS_SOCKADDR(lpNBaddr, NETBIOS_GROUP_NAME, 
			                    "VOICE_TALK_SERVER_NETBIOS", 0);
           pServSock->m_iAddrLen = lpProtBuf->iMaxSockAddr;
		   strcpy(strProts, "AF_NETBIOS");
           break;
       default:
		   strcpy(strProts, "Others");
		   break;
	}


    // Bind the server socket to address
    if(bind(pServSock->m_hSock, &pServSock->m_sockAddr, 
		    pServSock->m_iAddrLen) == SOCKET_ERROR)
    {
       // Error
       //AfxMessageBox("Can not bind the server socket"); 
       closesocket(pServSock->m_hSock);
       return FALSE;
    }

    // Get the socket local address
    if(getsockname(pServSock->m_hSock, &pServSock->m_sockAddr,
                   &pServSock->m_iAddrLen) == SOCKET_ERROR)
    {
       // Error
       //AfxMessageBox("Can not get the socket name"); 
       closesocket(pServSock->m_hSock);
       return FALSE;
    }

    // Listen on the socket
    if(listen(pServSock->m_hSock, 5) == SOCKET_ERROR)
    {
       // Error
       //AfxMessageBox("Can not listen the server socket"); 
       closesocket(pServSock->m_hSock);
       return FALSE;
    }

    // Set up window message to indicate connection
    if(WSAAsyncSelect(pServSock->m_hSock, hWnd,
                      VTM_CONNECTED, FD_ACCEPT) == SOCKET_ERROR)
    {
       // Error
       //AfxMessageBox("Can not select the server socket"); 
       closesocket(pServSock->m_hSock);
       return FALSE;
    }

    // OK
	return TRUE;
}
#endif
